@@ -50,6 +50,7 @@
 #include "headset.h"
 
 #include "obexserver.h"
+#include "obexclient.h"
 
 #define _(x) gettext(x)
 
@@ -278,8 +279,35 @@ device_clicked (GtkWidget *widget, GdkEventButton *e, gpointer data)
   gtk_menu_popup (GTK_MENU (device_menu), NULL, NULL, NULL, widget, 1, GDK_CURRENT_TIME);
 }
 
+const gchar *
+icon_name_for_class (int class)
+{
+  const gchar *pixbuf_name;
+
+  switch (class & 0x1f00)
+    {
+    case 0x100:
+      pixbuf_name = "computer";
+      break;
+    case 0x200:
+      pixbuf_name = "cellphone";
+      break;
+    case 0x300:
+      pixbuf_name = "network";
+      break;
+    case 0x600:
+      pixbuf_name = "printer";
+      break;
+    default:
+      pixbuf_name = "bt-logo";
+      break;
+    }
+
+  return pixbuf_name;
+}
+
 static gboolean
-run_scan (void)
+run_scan (gpointer data)
 {
   bdaddr_t bdaddr;
   inquiry_info *info = NULL;
@@ -289,6 +317,7 @@ run_scan (void)
   int i, dd;
   GtkWidget *w;
   GSList *iter;
+  const char *text;
 
   gdk_threads_enter ();
   w = bt_progress_dialog (_("Scanning for devices"), gpe_find_icon ("bt-logo"));
@@ -302,26 +331,16 @@ run_scan (void)
   num_rsp = hci_inquiry (dev_id, length, num_rsp, NULL, &info, flags);
   if (num_rsp < 0) 
     {
-      gdk_threads_enter ();
-      gtk_widget_destroy (w);
-      gpe_perror_box_nonblocking (_("Inquiry failed"));
-      gtk_widget_show_all (devices_window);
-      gdk_threads_leave ();
-      close (dev_id);
-      return FALSE;
+      text = _("Inquiry failed");
+      goto error;
     }
 
-  dd = hci_open_dev(0/*dev_id*/);
+  dd = hci_open_dev (0/*dev_id*/);
   if (dd < 0) 
     {
-      gdk_threads_enter ();
-      gtk_widget_destroy (w);
-      gpe_perror_box_nonblocking (_("HCI device open failed"));
-      gtk_widget_show_all (devices_window);
-      gdk_threads_leave ();
-      close (dev_id);
-      free(info);
-      return FALSE;
+      free (info);
+      text = _("HCI device open failed");
+      goto error;
     }
 
   for (i = 0; i < num_rsp; i++) 
@@ -355,24 +374,7 @@ run_scan (void)
       memcpy (&bd->bdaddr, &bdaddr, sizeof (bdaddr));
       bd->class = ((info+i)->dev_class[2] << 16) | ((info+i)->dev_class[1] << 8) | (info+i)->dev_class[0];
 
-      switch (bd->class & 0x1f00)
-	{
-	case 0x100:
-	  bd->pixbuf = gpe_find_icon ("computer");
-	  break;
-	case 0x200:
-	  bd->pixbuf = gpe_find_icon ("cellphone");
-	  break;
-	case 0x300:
-	  bd->pixbuf = gpe_find_icon ("network");
-	  break;
-	case 0x600:
-	  bd->pixbuf = gpe_find_icon ("printer");
-	  break;
-	default:
-	  bd->pixbuf = gpe_find_icon ("bt-logo");
-	  break;
-	}
+      bd->pixbuf = gpe_find_icon (icon_name_for_class (bd->class));
 
       gdk_pixbuf_ref (bd->pixbuf);
 
@@ -380,7 +382,7 @@ run_scan (void)
     }
   
   close (dd);
-  close (dev_id);
+  free (info);
 
   gdk_threads_enter ();
 
@@ -414,6 +416,14 @@ run_scan (void)
   gdk_threads_leave ();
 
   return TRUE;
+
+ error:
+  gdk_threads_enter ();
+  gtk_widget_destroy (w);
+  gpe_perror_box_nonblocking (text);
+  gtk_widget_show_all (devices_window);
+  gdk_threads_leave ();
+  return FALSE;
 }
 
 static void
@@ -452,11 +462,19 @@ sigchld_handler (int sig)
 
   if (p == hciattach_pid)
     {
-      hciattach_pid = 0;
-      if (radio_is_on)
+      if (WIFSIGNALED (status) && (WTERMSIG (status) == SIGHUP))
 	{
-	  gpe_error_box_nonblocking (_("hciattach died unexpectedly"));
-	  radio_off ();
+	  /* restart hciattach after hangup */
+	  hciattach_pid = fork_hciattach ();
+	}
+      else
+	{
+	  hciattach_pid = 0;
+	  if (radio_is_on)
+	    {
+	      gpe_error_box_nonblocking (_("hciattach died unexpectedly"));
+	      radio_off ();
+	    }
 	}
     }
   else if (p > 0)
@@ -490,7 +508,7 @@ main (int argc, char *argv[])
 {
   GtkWidget *window;
   GdkBitmap *bitmap;
-  GtkWidget *menu_remove;
+  GtkWidget *menu_remove, *menu_file;
   GtkTooltips *tooltips;
   int dd;
 
@@ -527,11 +545,13 @@ main (int argc, char *argv[])
   menu_radio_on = gtk_menu_item_new_with_label (_("Switch radio on"));
   menu_radio_off = gtk_menu_item_new_with_label (_("Switch radio off"));
   menu_devices = gtk_menu_item_new_with_label (_("Devices..."));
+  menu_file = gtk_menu_item_new_with_label (_("Send file ..."));
   menu_remove = gtk_menu_item_new_with_label (_("Remove from panel"));
 
   g_signal_connect (G_OBJECT (menu_radio_on), "activate", G_CALLBACK (radio_on), NULL);
   g_signal_connect (G_OBJECT (menu_radio_off), "activate", G_CALLBACK (radio_off), NULL);
   g_signal_connect (G_OBJECT (menu_devices), "activate", G_CALLBACK (show_devices), NULL);
+  g_signal_connect (G_OBJECT (menu_file), "activate", G_CALLBACK (send_file_dialog), NULL);
   g_signal_connect (G_OBJECT (menu_remove), "activate", G_CALLBACK (gtk_main_quit), NULL);
 
   if (! radio_is_on)
@@ -542,10 +562,12 @@ main (int argc, char *argv[])
 
   gtk_widget_show (menu_devices);
   gtk_widget_show (menu_remove);
+  gtk_widget_show (menu_file);
 
   gtk_menu_append (GTK_MENU (menu), menu_radio_on);
   gtk_menu_append (GTK_MENU (menu), menu_radio_off);
   gtk_menu_append (GTK_MENU (menu), menu_devices);
+  gtk_menu_append (GTK_MENU (menu), menu_file);
   gtk_menu_append (GTK_MENU (menu), menu_remove);
 
   if (gpe_load_icons (my_icons) == FALSE)
